/*
  file: pagination.js 

  functions for pagination of table data
  one page contains nrows_perpage rows
  one block contains npages
  one block of table data is loaded in the DOM
  one "page" = one element "tbody"
  only one page is made visible (style="display: table-row-group")
  the other pages ("tbody's") are not shown (style="display: none")
  max_rows = total number of rows
  i.e.: if we have 236 rows and npages= 10 and nrows_perpage = 10:
    total pages = 24 (last page contains only 6 rows)
    total blocks = 3 (last block contains only 4 pages)
  slow forward/rewind ('>'/'<'): per page
  fast forward/rewind ('>>'/'<<'): per block
  hidden paginators (<<,<,> or >>) are used for alignment.
*/

function collapse_oldpage(idfrom) {
  // hide the page (= tbody) where we came from
  tbody = "tbody"+idfrom;
  var p = document.getElementById(tbody);
  if(p.style.display == 'table-row-group') {
    p.style.display = 'none';
  }
}
      
function change_pagination(sort,id,npages,nrows_perpage,max_rows) {
  /*
  before displaying a new page get the pagination right
  we create a new div and use jquery's "replaceWith" to put the
  new div in place.
  The arguments:
  sort: sort order for the rows, needed as argument for show_page
  id: id of page which was clicked
  npages: number of pages to present at once (with only one page visible, the rest hidden)
  nrows_perpage: number of rows per page 
  max_rows: total number of rows in the database
  */
       
  var replace = '<div id="pagination">';
  if (id > npages) {
    // we need to make a "prevblock (<<)"
    // for example: if id = 24 and we have 10 pages per block (npages = 10) our previous block starts at page = 11
    //              24 - ((10 - 1) + 24%10) = 24 - (9 + 4) = 11
    //              if id = 20 and we have 10 pages per block (npages = 10) our previous block starts at page = 1
    //              id%npages == 0, so subtract with an extra npages: 20 - ((10 - 1) + 0 + 10) = 1
    prev = id-((npages - 1) + id%npages);
    if (id%npages == 0 ) {
      prev = prev - npages;
    }
    var replace = replace+'<a href="javascript:void(0);" style="color:blue" onmouseover="this.style.cursor=\'pointer\';" onclick="show_page(\''+sort+'\','+prev+','+id+','+npages+','+nrows_perpage+','+max_rows+');return false;"><<</a>\n';
  } else {
    // no prevblock, use a hidden prevblock for alignment
    var replace = replace+'<a style="visibility:hidden"><<</a>\n';
  }
  if (id > 1) {
    // we need to make a "prevpage"
    prev = id-1;
    var replace = replace+'<a href="javascript:void(0);" style="color:blue" onmouseover="this.style.cursor=\'pointer\';" onclick="show_page(\''+sort+'\','+prev+','+id+','+npages+','+nrows_perpage+','+max_rows+');return false;"><</a>\n';
  } else {
    // id = 1, no need for a prevpage, use a hidden prevpage for alignment
    var replace = replace+'<a style="visibility:hidden"><</a>\n';
  }
  if (id > npages) {
    // alway echo "page 1" when id > npages
    var replace = replace+'<a href="javascript:void(0);" style="color:blue" onmouseover="this.style.cursor=\'pointer\';" onclick="show_page(\''+sort+'\',1,'+id+','+npages+','+nrows_perpage+','+max_rows+');return false;">1</a> <span class="dots">...</span> ';
  }

  // get start (index) for our pages in the pagination
  // while we are here anyway, define our nextblock.
  if (id%npages == 0) {
    // we are at the end of our pagination
    // index starts at the beginning
    index = id - (npages - 1);
    nextblock = id + 1;
  } else {
    // index starts at the beginning
    index = id - (id%npages -1);
    nextblock = index + npages;
  }
  for (i=1;i<=npages;i++) {
    // we need keep an eye on the fact the last pages most likely won't fill up the whole pagination range
    // for example: if we have 37 pages and the pagination is done per 10 pages the
    //              last pagination range is from 31 - 37 (and not from 31 - 31 + 10).
    nrows = index*nrows_perpage;
    var restrows = max_rows - nrows;
    if (restrows > -nrows_perpage) {
      if (index == id) {
        // active page indicated with red
        // var replace = replace+'<a href="javascript:void(0);" id="page'+i+'" style="color:red" onclick="show_page(\''+sort+'\','+index+','+id+','+npages+','+nrows_perpage+','+max_rows+');return false;">'+index+'</a>\n';
        var replace = replace+'<a href="javascript:void(0);" id="page'+i+'" style="color:red;cursor: default">'+index+'</a>\n';
      } else {
        // non active page indicated with black
        var replace = replace+'<a href="javascript:void(0);" title="make hidden page visible" id="page'+i+'" style="color:black" onclick="show_page(\''+sort+'\','+index+','+id+','+npages+','+nrows_perpage+','+max_rows+');return false;">'+index+'</a>\n';
      }
      index++;
    }
  }
  // correct for looping one to far...
  if (restrows <= 0) {
    index--;
  }

  // determine last page-id
  lastpageid = Math.ceil(max_rows/nrows_perpage);
  // always show last page-id when not in last block
  nlastpages = lastpageid%npages;
  if (lastpageid%npages == 0) {
    nlastpages = npages;
  }
  if (id < lastpageid - nlastpages + 1) {
    // always show last page-id when id < lastpageid -nlastpages +1
    // i.e. if lastpageid = 70,  id = 60 and npages = 10 then (60 < 70 -10 +1) is true: show lastpageid
    var replace = replace+' <span class="dots">...</span> <a href="javascript:void(0);" style="color:blue" onmouseover="this.style.cursor=\'pointer\';" onclick="show_page(\''+sort+'\','+lastpageid+','+id+','+npages+','+nrows_perpage+','+max_rows+');return false;">'+lastpageid+' </a>';
  }
  if (id < index) {
    // we still can go slow forward, make nextpage
    next=id+1;
    var replace = replace+'<a href="javascript:void(0);" style="color:blue" onmouseover="this.style.cursor=\'pointer\';" onclick="show_page(\''+sort+'\','+next+','+id+','+npages+','+nrows_perpage+','+max_rows+');return false;">></a>\n';
  } else {
    // no need for a nextpage, use a hidden nextpage for alignment
    var replace = replace+'<a style="visibility:hidden">></a>\n';
  }
  if (nrows < max_rows) {
    // we did not enter the last blcok, we can still go fast forward, make nextblock
    var replace = replace+'<a href="javascript:void(0);" style="color:blue" onmouseover="this.style.cursor=\'pointer\';" onclick="show_page(\''+sort+'\','+nextblock+','+id+','+npages+','+nrows_perpage+','+max_rows+');return false;">>></a>\n';
  } else {
    // no need for a nextblock, use a hidden nextblock for alignment
    var replace = replace+'<a style="visibility:hidden"">>></a>\n';
  }
    
  // Calculate visible startrow and endrow for displaying "Results: x - y of $max_rows"
  first = ((id-1)*nrows_perpage) + 1;
  last = first + (nrows_perpage -1); // 1 - 10 == 1 - 1+9 
  if (last > max_rows) {
    // we are at the end of the rows
    last = max_rows;
  }
  result = '<span id="results">Results: '+first+' - '+last+' of '+max_rows+'</span>';
  replace = replace+result+'</div>';
  $("#pagination").replaceWith(replace);
}

function show_page(sort,id,idfrom,npages,nrows,max_rows) {
  // change the div pagination
  // if max_rows = -1 (only at the start) we need to get it
  if (max_rows == -1) {
    var max_rows = $.ajax({ url: "max_rows.php", async: false }).responseText;
  }
  change_pagination(sort,id,npages,nrows,max_rows);

  // change the old page from visible (style.display == 'table-row-group') to hidden (style.display = 'none')
  // if idfrom == -1 (start) there's nothing to collapse
  if (idfrom != -1) {
    collapse_oldpage(idfrom);
  }

  // calculate rest if id == lastpageid
  rest = npages;
  lastpageid = Math.ceil(max_rows/nrows);
  if ((id == lastpageid) && (id%npages != 0)) {
    rest = id%npages;
  }
  // get new rows from the database
  // do this only when we press next ('>', or '>>') or previous ('<', or '<<') or if we press the first or last pageid and we come from another block
  if ( (id%npages == 1 && (id > idfrom || id <= (idfrom - npages))) || (id%npages == 0 && (idfrom - id == 1)) || (id == lastpageid && (id - rest >= idfrom)) ) {
    // we need to get new rows from the database, show a "loading" message.
    $("#main").replaceWith("<div id=\"main\">Loading new data...</div>");
    // get new rows
    // determine begin row for first page (id = clicked page)
    if (id == lastpageid) {
      if (id%npages == 0) {
        from = (id - npages) * nrows; 
      } else {
        from = (id - (id%npages)) * nrows; 
      }
    } else {
      if (id%npages == 0) {
        from = (id - npages) * nrows; 
      } else {
        // we pressed 1, nextpage or nextblock.
        from = (id - 1) * nrows;
      }
    }
    // we need npages*nrows to display
    to = npages * nrows;
    // start the new div
    var newrows = '<div id="main" title="data area">';
    // get all rows using an ajax-call to get_rows.php
    // get_rows.php return the table with the new rows
    // get_rows.php makes the correct page (=tbody) visible
    var rows = $.ajax({ url: "get_rows.php",
      data: "sort="+sort+"&id="+id+"&npages="+npages+"&nrows="+nrows+"&max_rows="+max_rows+"&from="+from+"&to="+to,
      async: false
    }).responseText;
    // close our div "main"
    newrows = newrows+rows+'</div>';
    // use jqeury's "replaceWith" to replace the existing div with id = "main" with the new data
    $("#main").replaceWith(newrows);
  } else { 
    // we did not load new data
    // we already have hidden the old page (with collapse_oldpage(idfrom))
    // make page (= tbody) visible with id which we clicked in the navigation div
    $("#tbody"+id).attr("style", "display:table-row-group");
  }
}
   
function sorted_rows(sort,startid,npages,nrows,max_rows) {
  $("#main").replaceWith("<div id=\"main\">Loading new data...</div>");
  change_pagination(sort,startid,npages,nrows,max_rows);
  from = (startid - 1) * nrows;
  to = npages * nrows;   
  var newrows = '<div id="main">';               
  var rows = $.ajax({ url: "get_rows.php",
    data: "sort="+sort+"&id="+startid+"&npages="+npages+"&nrows="+nrows+"&max_rows="+max_rows+"&from="+from+"&to="+to,
    async: false
  }).responseText;
  newrows = newrows+rows+'</div>';
  $("#main").replaceWith(newrows);
}
